--------------------------------------------------------------------------------
-- Title   : Register Mutiplexer
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : Register multiplexer
--  SW 5-4-3-2-1-0
--     1-1-1-1-1-1 : board test
--     0-0-0-0-1-1 : LINAC monitoring
--     0-0-0-0-1-0 : LINAC multipulse
--     0-0-0-0-0-1 : top-up trigger
--     0-0-0-0-0-0 : duplication
--------------------------------------------------------------------------------
-- File     : cPCI_registerMux.vhdl
-- Revision : x.x.x
-- Created  : November 17, 2013
-- Updated  : November 27, 2013
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.registers_init.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity cPCI_registerMux is
  port (
    -- cPCI registers
    p_wRegister0  : in  std_logic_vector (31 downto 0);
    p_wRegister1  : in  std_logic_vector (31 downto 0);
    p_wRegister2  : in  std_logic_vector (31 downto 0);
    p_wRegister3  : in  std_logic_vector (31 downto 0);
    p_wRegister4  : in  std_logic_vector (31 downto 0);
    p_wRegister5  : in  std_logic_vector (31 downto 0);
    p_wRegister6  : in  std_logic_vector (31 downto 0);
    p_wRegister7  : in  std_logic_vector (31 downto 0);
    p_wRegister8  : in  std_logic_vector (31 downto 0);
    p_wRegister9  : in  std_logic_vector (31 downto 0);
    p_wRegister10 : in  std_logic_vector (31 downto 0);
    p_wRegister11 : in  std_logic_vector (31 downto 0);
    p_wRegister12 : in  std_logic_vector (31 downto 0);
    p_wRegister13 : in  std_logic_vector (31 downto 0);

    p_wRegister0_init  : out std_logic_vector (31 downto 0);
    p_wRegister1_init  : out std_logic_vector (31 downto 0);
    p_wRegister2_init  : out std_logic_vector (31 downto 0);
    p_wRegister3_init  : out std_logic_vector (31 downto 0);
    p_wRegister4_init  : out std_logic_vector (31 downto 0);
    p_wRegister5_init  : out std_logic_vector (31 downto 0);
    p_wRegister6_init  : out std_logic_vector (31 downto 0);
    p_wRegister7_init  : out std_logic_vector (31 downto 0);
    p_wRegister8_init  : out std_logic_vector (31 downto 0);
    p_wRegister9_init  : out std_logic_vector (31 downto 0);
    p_wRegister10_init : out std_logic_vector (31 downto 0);
    p_wRegister11_init : out std_logic_vector (31 downto 0);
    p_wRegister12_init : out std_logic_vector (31 downto 0);
    p_wRegister13_init : out std_logic_vector (31 downto 0);

    p_rRegister0  : out std_logic_vector (31 downto 0);
    p_rRegister1  : out std_logic_vector (31 downto 0);
    p_rRegister2  : out std_logic_vector (31 downto 0);
    p_rRegister3  : out std_logic_vector (31 downto 0);
    p_rRegister4  : out std_logic_vector (31 downto 0);
    p_rRegister5  : out std_logic_vector (31 downto 0);
    p_rRegister6  : out std_logic_vector (31 downto 0);
    p_rRegister7  : out std_logic_vector (31 downto 0);
    p_rRegister8  : out std_logic_vector (31 downto 0);
    p_rRegister9  : out std_logic_vector (31 downto 0);
    p_rRegister10 : out std_logic_vector (31 downto 0);
    p_rRegister11 : out std_logic_vector (31 downto 0);
    p_rRegister12 : out std_logic_vector (31 downto 0);
    p_rRegister13 : out std_logic_vector (31 downto 0);

    -- Mux in / out
    p_mux_wRegister0  : out std_logic_vector (31 downto 0);
    p_mux_wRegister1  : out std_logic_vector (31 downto 0);
    p_mux_wRegister2  : out std_logic_vector (31 downto 0);
    p_mux_wRegister3  : out std_logic_vector (31 downto 0);
    p_mux_wRegister4  : out std_logic_vector (31 downto 0);
    p_mux_wRegister5  : out std_logic_vector (31 downto 0);
    p_mux_wRegister6  : out std_logic_vector (31 downto 0);
    p_mux_wRegister7  : out std_logic_vector (31 downto 0);
    p_mux_wRegister8  : out std_logic_vector (31 downto 0);
    p_mux_wRegister9  : out std_logic_vector (31 downto 0);
    p_mux_wRegister10 : out std_logic_vector (31 downto 0);
    p_mux_wRegister11 : out std_logic_vector (31 downto 0);
    p_mux_wRegister12 : out std_logic_vector (31 downto 0);
    p_mux_wRegister13 : out std_logic_vector (31 downto 0);

    p_mux_rRegister0  : in  std_logic_vector (31 downto 0);
    p_mux_rRegister1  : in  std_logic_vector (31 downto 0);
    p_mux_rRegister2  : in  std_logic_vector (31 downto 0);
    p_mux_rRegister3  : in  std_logic_vector (31 downto 0);
    p_mux_rRegister4  : in  std_logic_vector (31 downto 0);
    p_mux_rRegister5  : in  std_logic_vector (31 downto 0);
    p_mux_rRegister6  : in  std_logic_vector (31 downto 0);
    p_mux_rRegister7  : in  std_logic_vector (31 downto 0);
    p_mux_rRegister8  : in  std_logic_vector (31 downto 0);
    p_mux_rRegister9  : in  std_logic_vector (31 downto 0);
    p_mux_rRegister10 : in  std_logic_vector (31 downto 0);
    p_mux_rRegister11 : in  std_logic_vector (31 downto 0);
    p_mux_rRegister12 : in  std_logic_vector (31 downto 0);
    p_mux_rRegister13 : in  std_logic_vector (31 downto 0);

    p_mux_csw         : in  std_logic_vector (15 downto 0); -- chip select of internal write transfert
    p_mux_csr         : in  std_logic_vector (15 downto 0); -- chip select of internal read transfert
    p_csw             : out std_logic_vector (15 downto 0);
    p_csr             : out std_logic_vector (15 downto 0);

    -- Congig pin
    p_mux_SWsel       : in  std_logic_vector (5 downto 0)
  );
end entity cPCI_registerMux;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_cPCI_registerMux of cPCI_registerMux is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- Components
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- Duplication write
  with p_mux_SWsel select
    p_mux_wRegister0 <=
      p_wRegister0 when "000000",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_mux_wRegister1 <=
      p_wRegister1 when "000000",
      X"00000000"  when OTHERS;

  -- Top-up write
  with p_mux_SWsel select
    p_mux_wRegister2 <=
      p_wRegister0 when "000001",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_mux_wRegister3 <=
      p_wRegister1 when "000001",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_mux_wRegister4 <=
      p_wRegister2 when "000001",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_mux_wRegister5 <=
      p_wRegister3 when "000001",
      X"00000000"  when OTHERS;

  -- LINAC SPM multi-pulses write
  with p_mux_SWsel select
    p_mux_wRegister6 <=
      p_wRegister0 when "000010",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_mux_wRegister7 <=
      p_wRegister1 when "000010",
      X"00000000"  when OTHERS;

  -- LINAC monitoring write
  with p_mux_SWsel select
    p_mux_wRegister8 <=
      p_wRegister0 when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_mux_wRegister9 <=
      p_wRegister1 when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_mux_wRegister10 <=
      p_wRegister2 when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_mux_wRegister11 <=
      p_wRegister3 when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_mux_wRegister12 <=
      p_wRegister4 when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_mux_wRegister13 <=
      p_wRegister5 when "000011",
      X"00000000"  when OTHERS;

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  -- Write registers init values mux
  with p_mux_SWsel select
    p_wRegister0_init <=
      c_wRegister0_init when "000000",
      c_wRegister2_init when "000001",
      c_wRegister6_init when "000010",
      c_wRegister8_init when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_wRegister1_init <=
      c_wRegister1_init when "000000",
      c_wRegister3_init when "000001",
      c_wRegister7_init when "000010",
      c_wRegister9_init when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_wRegister2_init <=
      c_wRegister4_init  when "000001",
      c_wRegister10_init when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_wRegister3_init <=
      c_wRegister5_init  when "000001",
      c_wRegister11_init when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_wRegister4_init <=
      c_wRegister12_init when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_wRegister5_init <=
      c_wRegister13_init when "000011",
      X"00000000"  when OTHERS;

  p_wRegister6_init  <= X"00000000";
  p_wRegister7_init  <= X"00000000";
  p_wRegister8_init  <= X"00000000";
  p_wRegister9_init  <= X"00000000";
  p_wRegister10_init <= X"00000000";
  p_wRegister11_init <= X"00000000";
  p_wRegister12_init <= X"00000000";
  p_wRegister13_init <= X"00000000";

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  -- Read mux
  with p_mux_SWsel select
    p_rRegister0 <=
      p_mux_rRegister0 when "000000",
      p_mux_rRegister2 when "000001",
      p_mux_rRegister6 when "000010",
      p_mux_rRegister8 when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_rRegister1 <=
      p_mux_rRegister1 when "000000",
      p_mux_rRegister3 when "000001",
      p_mux_rRegister7 when "000010",
      p_mux_rRegister9 when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_rRegister2 <=
      p_mux_rRegister4  when "000001",
      p_mux_rRegister10 when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_rRegister3 <=
      p_mux_rRegister5  when "000001",
      p_mux_rRegister11 when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_rRegister4 <=
      p_mux_rRegister12 when "000011",
      X"00000000"  when OTHERS;

  with p_mux_SWsel select
    p_rRegister5 <=
      p_mux_rRegister13 when "000011",
      X"00000000"  when OTHERS;

  p_rRegister6  <= X"00000000";
  p_rRegister7  <= X"00000000";
  p_rRegister8  <= X"00000000";
  p_rRegister9  <= X"00000000";
  p_rRegister10 <= X"00000000";
  p_rRegister11 <= X"00000000";
  p_rRegister12 <= X"00000000";
  p_rRegister13 <= X"00000000";

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  p_csw(0)  <= p_mux_csw(0);  -- cmd register

  -- Duplication write strobe mux
  with p_mux_SWsel select
    p_csw(1) <=
      p_mux_csw(1) when "000000",
      '0'  when OTHERS;

  with p_mux_SWsel select
    p_csw(2) <=
      p_mux_csw(2) when "000000",
      '0'  when OTHERS;

  -- Top-up write strobe mux
  with p_mux_SWsel select
    p_csw(3) <=
      p_mux_csw(1) when "000001",
      '0'  when OTHERS;

  with p_mux_SWsel select
    p_csw(4) <=
      p_mux_csw(2) when "000001",
      '0'  when OTHERS;

  with p_mux_SWsel select
    p_csw(5) <=
      p_mux_csw(3) when "000001",
      '0'  when OTHERS;

  with p_mux_SWsel select
    p_csw(6) <=
      p_mux_csw(4) when "000001",
      '0'  when OTHERS;

  -- LINAC multipulse write strobe mux
  with p_mux_SWsel select
    p_csw(7) <=
      p_mux_csw(1) when "000010",
      '0'  when OTHERS;

  with p_mux_SWsel select
    p_csw(8) <=
      p_mux_csw(2) when "000010",
      '0'  when OTHERS;

  -- LINAC monitoring write strobe mux
  with p_mux_SWsel select
    p_csw(9) <=
      p_mux_csw(1) when "000011",
      '0'  when OTHERS;

  with p_mux_SWsel select
    p_csw(10) <=
      p_mux_csw(2) when "000011",
      '0'  when OTHERS;

  with p_mux_SWsel select
    p_csw(11) <=
      p_mux_csw(3) when "000011",
      '0'  when OTHERS;

  with p_mux_SWsel select
    p_csw(12) <=
      p_mux_csw(4) when "000011",
      '0'  when OTHERS;

  with p_mux_SWsel select
    p_csw(13) <=
      p_mux_csw(5) when "000011",
      '0'  when OTHERS;

  with p_mux_SWsel select
    p_csw(14) <=
      p_mux_csw(6) when "000011",
      '0'  when OTHERS;

  p_csw(15) <= p_mux_csw(15);

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  p_csr(1)  <= p_mux_csr(1);
  p_csr(2)  <= p_mux_csr(2);
  p_csr(3)  <= p_mux_csr(3);
  p_csr(4)  <= p_mux_csr(4);
  p_csr(5)  <= p_mux_csr(5);
  p_csr(6)  <= p_mux_csr(6);
  p_csr(7)  <= p_mux_csr(7);
  p_csr(8)  <= p_mux_csr(8);
  p_csr(9)  <= p_mux_csr(9);
  p_csr(10) <= p_mux_csr(10);
  p_csr(11) <= p_mux_csr(11);
  p_csr(12) <= p_mux_csr(12);
  p_csr(13) <= p_mux_csr(13);
  p_csr(14) <= p_mux_csr(14);
  p_csr(15) <= p_mux_csr(15);

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

end architecture rtl_cPCI_registerMux;

